"""
Constants used across the Reality Defender SDK
"""

# Default API endpoint
DEFAULT_API_ENDPOINT = "https://api.prd.realitydefender.xyz"

# API paths
API_PATHS = {
    "SIGNED_URL": "/api/files/aws-presigned",
    "MEDIA_RESULT": "/api/media/users",
}

# Default polling interval in milliseconds
DEFAULT_POLLING_INTERVAL = 2000

# Default timeout in milliseconds (1 minute)
DEFAULT_TIMEOUT = 60000

# Default maximum polling attempts
DEFAULT_MAX_ATTEMPTS = 30
